/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: CSTextI.h,v $
 * Revision 1.1.8.5  1994/07/25  13:49:54  daniel
 * 	fix for 9844: linkage problem in C++
 * 	[1994/07/25  13:41:51  daniel]
 *
 * Revision 1.1.8.4  1994/06/01  21:11:22  drk
 * 	CR 6873: Resolve P.h name conflicts.
 * 	[1994/06/01  16:57:33  drk]
 * 
 * Revision 1.1.8.3  1994/05/25  15:26:23  dick
 * 	Make end of output line scanning work like XmText - CR 9278
 * 	[1994/05/25  15:24:19  dick]
 * 
 * Revision 1.1.8.2  1994/05/24  21:55:05  dick
 * 	Clean up the macros for accessing subobject methods
 * 	[1994/05/24  21:41:39  dick]
 * 
 * Revision 1.1.8.1  1994/05/11  11:56:11  dick
 * 	Make all character traversal work with widget fontlist
 * 	[1994/05/11  11:50:25  dick]
 * 
 * Revision 1.1.6.8  1994/04/28  21:38:51  dick
 * 	Eliminate extraneous global procedures
 * 	[1994/04/28  14:36:06  dick]
 * 
 * Revision 1.1.6.7  1994/04/27  13:09:30  mryan
 * 	Change data types
 * 	[1994/04/27  12:45:19  mryan]
 * 
 * Revision 1.1.6.6  1994/04/15  21:17:15  drk
 * 	CR 8601: Misc. gcc warnings.
 * 	[1994/04/15  20:59:17  drk]
 * 
 * Revision 1.1.6.5  1994/04/12  22:43:33  dick
 * 	Add byte/char count converters
 * 	[1994/04/12  22:43:15  dick]
 * 
 * Revision 1.1.6.4  1994/04/12  12:46:39  dick
 * 	Add default TextType to widget
 * 	[1994/04/12  12:45:24  dick]
 * 
 * Revision 1.1.6.3  1994/03/25  01:34:09  dick
 * 	Add the support for *-tab-beginning actions
 * 	[1994/03/25  01:30:17  dick]
 * 
 * Revision 1.1.6.2  1994/03/18  20:30:31  dick
 * 	Eliminate unused direction macros
 * 	[1994/03/18  20:29:50  dick]
 * 
 * Revision 1.1.6.1  1994/03/04  22:21:09  drk
 * 	Hide _Xm routines from public view.
 * 	[1994/03/04  21:54:40  drk]
 * 
 * Revision 1.1.4.3  1993/09/16  02:28:22  motifdec
 * 	Some performance and restructuring with new XmString structs
 * 	[1993/09/16  02:14:32  motifdec]
 * 
 * Revision 1.1.4.2  1993/09/14  22:19:46  drk
 * 	ANSIfication of sources -- use memfoo instead of bfoo
 * 	[1993/09/14  22:15:30  drk]
 * 
 * Revision 1.1.2.10  1993/07/23  02:46:30  yak
 * 	Expended copyright marker
 * 	[1993/07/23  01:11:18  yak]
 * 
 * Revision 1.1.2.9  1993/07/19  20:21:40  motifdec
 * 	More Alpha fixes
 * 	[1993/07/19  19:47:57  motifdec]
 * 
 * Revision 1.1.2.8  1993/07/09  15:24:00  motifdec
 * 	More fixes to pass Alpha Acceptance tests
 * 	[1993/07/09  15:21:37  motifdec]
 * 
 * Revision 1.1.2.7  1993/07/02  17:20:16  motifdec
 * 	Move some source specific protos to CSTextSrcP.h, remove dup. errors,
 * 	rename text_path to string_direction.
 * 	[1993/07/02  17:11:04  motifdec]
 * 
 * Revision 1.1.2.6  1993/06/25  18:46:19  motifdec
 * 	Removing refs to bcopy, etc. headers
 * 	[1993/06/25  18:42:20  motifdec]
 * 
 * Revision 1.1.2.5  1993/06/16  20:18:34  motifdec
 * 	June 16 code drop from digital
 * 	[1993/06/16  18:59:26  motifdec]
 * 
 * $EndLog$
 */
/*
 */

#ifndef _XmCSTextI_h
#define _XmCSTextI_h

#include <limits.h>	/* For INT_MAX */
#include <stdlib.h>	/* for wchar_t */

#include <Xm/XmosP.h>
#include <Xm/CSTextP.h>
#include <Xm/CSTextSrcP.h>
#include <Xm/CSTextOutP.h>
#include <Xm/CSTextInP.h>


#ifdef __cplusplus
extern "C" {
#endif

typedef enum {CSWORDSELECT, CSTEXTWRAP} CSScanType;

#define IN_LOSE_SELECTION -999

#define ALLOC_DEFAULT_CHARSET( charset_ptr )  \
		  { charset_ptr = XtMalloc(strlen(XmSTRING_ISO8859_1)); \
	strncpy ( charset_ptr, XmSTRING_ISO8859_1, strlen(XmSTRING_ISO8859_1));}

#define CSCvtStatusOK        1
#define CSCvtStatusDataLoss  2
#define CSCvtStatusFail      3

#define BEEP(w)			( XBell ( XtDisplay( w ), 0 ) )

#define CastIT(w)		((XmCSTextWidget)(w))

#define Output(w)              	(CastIT (w)->cstext.output)
#define Input(w)               	(CastIT (w)->cstext.input)
#define CSTextWidgetIndex(w)    (CastIT (w)->cstext.widget_index)
#define Source(w)		(CastIT (w)->cstext.source)
#define SourceData(w) \
		((XmCSTextSourceData)(((XmCSTextSource)(Source(w)))->data))

#define OutputMethod(w, method) (Output(w)->method)
#define InputMethod(w, method) (Input(w)->method)
#define SourceMethod(w, method) (Source(w)->method)

#define TopChar(w)		(CastIT (w)->cstext.top_character)
#define BotPos(w)              	(CastIT (w)->cstext.bottom_position)
#define DestPos(w)              (CastIT (w)->cstext.dest_position)
#define CursorPos(w)           	(CastIT (w)->cstext.cursor_position)

#define ActivateCB(w)          	(CastIT (w)->cstext.activate_callback)
#define FocusCB(w)             	(CastIT (w)->cstext.focus_callback)
#define LoseFocusCB(w)         	(CastIT (w)->cstext.losing_focus_callback)
#define ValueCB(w)             	(CastIT (w)->cstext.value_changed_callback)
#define MotionCB(w)            	(CastIT (w)->cstext.motion_verify_callback)
#define ModifyCB(w)            	(CastIT (w)->cstext.modify_verify_callback)
#define GainPrimaryCB(w)       	(CastIT (w)->cstext.gain_primary_callback)
#define LosePrimaryCB(w)       	(CastIT (w)->cstext.lose_primary_callback)

#define NeedRedisplay(w)       	(CastIT (w)->cstext.needs_redisplay)
#define SetNeedRedisplay(w)    	(NeedRedisplay (w) = TRUE)
#define ResetNeedRedisplay(w)  	(NeedRedisplay (w) = FALSE)

#define PendingVScroll(w)      	(CastIT (w)->cstext.pending_vertical_scroll)
#define PendingHScroll(w)      	(CastIT (w)->cstext.pending_horiz_scroll)

#define CSTextLines(w)          (CastIT (w)->cstext.source->data->source_lines)

#define CSTextMaxLength(w)      (CastIT (w)->cstext.max_length)
#define CSTextLength(w)         (CastIT (w)->cstext.source->data->length)
#define CSTextHasSelection(w)   (CastIT (w)->cstext.source->data->hasselection)
#define CSTextSelLeft(w)        (CastIT (w)->cstext.source->data->left)
#define CSTextSelRight(w)       (CastIT (w)->cstext.source->data->right)

#define CSTextMarginWidth(w)    (CastIT (w)->cstext.margin_width)
#define CSTextMarginHeight(w)   (CastIT (w)->cstext.margin_height)
#define CSTextHasFocus(w)       (CastIT (w)->cstext.has_focus)

#define CSTextOutputCreate(w)   (CastIT (w)->cstext.output_create)
#define CSTextInputCreate(w)    (CastIT (w)->cstext.input_create)
#define CSTextDisableDepth(w)   (CastIT (w)->cstext.disable_depth)
#define INCR_DISABLEDEPTH(w)	(CastIT (w)->cstext.disable_depth++)
#define DECR_DISABLEDEPTH(w)   	(CastIT (w)->cstext.disable_depth--)
#define CSTextInRedisplay(w)    (CastIT (w)->cstext.in_redisplay)
#define CSTextTraversed(w)      (CastIT (w)->cstext.traversed)
#define CSTextAddMode(w)        (CastIT (w)->cstext.add_mode)
#define CSTextEditMode(w)       (CastIT (w)->cstext.edit_mode)
#define CSTextEditable(w)       (CastIT (w)->cstext.editable)
#define CSTextEditingPath(w)    (CastIT (w)->cstext.editing_path)
#define CSTextPath(w)           (CastIT (w)->cstext.string_direction)
#define CSTextValue(w)	       	(CastIT (w)->cstext.value)
#define CSTextNewTop(w)	       	(CastIT (w)->cstext.new_top)
#define CSTextAutoShowCursorPos(w) (CastIT (w)->cstext.auto_show_cursor_position)
#define CSTextDefaultCharSet(w) (CastIT (w)->cstext.def_charset)
#define CSTextDefaultTextType(w) (CastIT (w)->cstext.def_text_type)

#define CSTextVerifyBell(w)     (CastIT (w)->cstext.verify_bell)

#define CSTextPrimSelTime(w)	(SourceData(w)->prime_time)

#define CSTextDestSelTime(w)	(CST_InputData(w)->dest_time)
#define CSTextSecSelTime(w)	(CST_InputData(w)->sec_time)
#define CSTextHasDestination(w)	(CST_InputData(w)->has_destination)


/*
** FORWARD DECLARATIONS
*/

#ifdef _NO_PROTO
extern void _XmCSTextEnableRedisplay ();
extern void _XmCSTextDisableRedisplay ();
extern void _XmCSTextConvertCallback();
extern void _XmCSTextDestinationCallback();
extern void _XmCSTextInputCreate ();
extern void _XmCSTextOutputCreate ();
extern void _XmCSTextSetSelection ();
extern void _XmCSTextScanPositions ();
extern Boolean _XmCSTextIsScanBreak ();
extern Boolean _XmCSTextIsWhiteSpace ();
extern void _XmCSTextScanPreviousParagraph ();
extern void _XmCSTextScanNextParagraph ();
extern void _XmCSTextScanWord ();
extern void _XmCSTextScanNextTab ();
extern void _XmCSTextScanPrevTab ();
extern Boolean _XmCSTextScanWordLimits ();
extern void _XmCSTextScanEndOfLine ();
extern void _XmCSTextScanStartOfLine ();
extern void _XmCSTextScanNextLine ();
extern void _XmCSTextScanPrevLine ();
extern void _XmCSTextScanAll ();
extern Boolean _XmCSTextGetSelection ();
extern Boolean _XmCSTextNavNextChar ();
extern Boolean _XmCSTextNavPrevChar ();
extern Boolean _XmCSTextEqualLocation ();
extern Boolean _XmCSTextEqualSegment ();
extern XmCSTextLine _XmCSTextGetFirstLine ();
extern XmCSTextLine _XmCSTextGetLastLine ();
extern XmCSTextLine _XmCSTextGetNextLine ();
extern XmCSTextLine _XmCSTextGetPrevLine ();
extern _XmStringEntry _XmCSTextGetFirstSegment ();
extern _XmStringEntry _XmCSTextGetLastSegment ();
extern _XmStringEntry _XmCSTextGetNextSegment ();
extern _XmStringEntry _XmCSTextGetPrevSegment ();
extern int _XmCSTextNumLines ();
extern int _XmCSBytesPerChar ();
extern Boolean _XmCSTextGetSel2 ();
extern void _XmCvtCSTextToCSInit();
extern void _XmCvtCSTextToCSEnd();
extern void _XmCSTextInputGetSecResData();
extern Boolean _XmCSTextSetDestinationSelection ();
extern Boolean _XmCSTextSetSel2();
extern int _XmCSTextCharToByteCount();
extern int _XmCSTextByteToCharCount();
extern void _XmCSTextFreeContextData();
#else

extern void _XmCSTextEnableRedisplay (
			Widget		widget);
extern void _XmCSTextDisableRedisplay (
			Widget		widget,
#if NeedWidePrototypes
			int		losesbackingstore);
#else
			Boolean		losesbackingstore);
#endif /* NeedWidePrototypes */

extern void _XmCSTextConvertCallback(
			Widget			w,
			XtPointer		ignore,
                        XmConvertCallbackStruct	*cs);
extern void _XmCSTextDestinationCallback(
			Widget				w,
			XtPointer			closure,
			XmDestinationCallbackStruct	*ds);

extern void _XmCSTextInputCreate (
					XmCSTextWidget widget,
					ArgList args,
					Cardinal num_args );
extern void _XmCSTextOutputCreate (
					XmCSTextWidget widget,
					ArgList args,
					Cardinal num_args );
extern void _XmCSTextSetSelection (
				 	XmCSTextWidget widget,
					XmTextPosition left,
					XmTextPosition right,
					Time time,
					XmCSTextSelectionType selType );
extern void _XmCSTextScanPositions (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextScanDirection direction,
					int count,
					XmTextPosition *new_position );
extern Boolean _XmCSTextIsScanBreak (
					Widget widget,
					CSTextLocation location,
					int scan_direction,
					CSScanType scan_type );
extern Boolean _XmCSTextIsWhiteSpace (
					Widget widget,
					CSTextLocation location );
extern void _XmCSTextScanPreviousParagraph (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanNextParagraph (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanWord (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextScanDirection direction,
					XmTextPosition *new_position );

extern void _XmCSTextScanNextTab (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanPrevTab (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );

extern Boolean _XmCSTextScanWordLimits (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *first_position,
					XmTextPosition *last_position );
extern void _XmCSTextScanEndOfLine (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position,
#if NeedWidePrototypes
				        int include
#else
				        Boolean include
#endif
				    );
extern void _XmCSTextScanStartOfLine (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanNextLine (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanPrevLine (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextPosition *new_position );
extern void _XmCSTextScanAll (
					XmCSTextWidget widget,
					XmTextPosition position,
					XmTextScanDirection direction,
					XmTextPosition *new_position );
extern Boolean _XmCSTextGetSelection (
					XmCSTextWidget widget,
					XmTextPosition *left,
					XmTextPosition *right );
extern Boolean _XmCSTextNavNextChar (
				        XmCSTextWidget widget,
					CSTextLocation location );
extern Boolean _XmCSTextNavPrevChar (
				        XmCSTextWidget widget,
					CSTextLocation location );
extern Boolean _XmCSTextEqualLocation (
					CSTextLocation a,
					CSTextLocation b );
extern Boolean _XmCSTextEqualSegment (
					_XmStringEntry a,
					_XmStringEntry b );
extern XmCSTextLine _XmCSTextGetFirstLine (
					XmCSTextWidget widget );
extern XmCSTextLine _XmCSTextGetLastLine (
					XmCSTextWidget widget );
extern XmCSTextLine _XmCSTextGetNextLine (
					XmCSTextLine line );
extern XmCSTextLine _XmCSTextGetPrevLine (
					XmCSTextLine line );
extern _XmStringEntry _XmCSTextGetFirstSegment (
					XmCSTextLine line );
extern _XmStringEntry _XmCSTextGetLastSegment (
					XmCSTextLine line );
extern _XmStringEntry _XmCSTextGetNextSegment (
					_XmStringEntry segment );
extern _XmStringEntry _XmCSTextGetPrevSegment (
					_XmStringEntry segment );
extern int _XmCSTextNumLines (		XmCSTextWidget widget );
extern int _XmCSBytesPerChar (		char * charset );
extern Boolean _XmCSTextGetSel2 ( 	XmCSTextInputData data ,
					XmTextPosition *left ,
					XmTextPosition *right );
extern void _XmCvtCSTextToCSInit (	XmCSTextCvtContext context );
extern void _XmCvtCSTextToCSEnd (	XmCSTextCvtContext context );
extern void _XmCSTextInputGetSecResData (
					XmSecondaryResourceData *secResDataRtn);
extern Boolean _XmCSTextSetDestinationSelection (
					Widget w,
					XmTextPosition position,
#if NeedWidePrototypes
					int disown,
#else
					Boolean disown,
#endif /* NeedWidePrototypes */
					Time set_time );

extern Boolean _XmCSTextSetSel2 (	XmCSTextInputData data ,
					XmTextPosition left ,
					XmTextPosition right ,
					Time time );

extern int _XmCSTextByteToCharCount (	XmFontListEntry entry,
					XmTextType text_type,
					char *buf,
					int byte_count_in );

extern int _XmCSTextCharToByteCount (	XmFontListEntry entry,
					XmTextType text_type,
					char *buf,
					int char_count_in );
extern void _XmCSTextFreeContextData( 
                        Widget w,
                        XtPointer clientData,
                        XtPointer callData) ;
#endif /* _NO_PROTO */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /* _XmCSTextI_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */
