/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: ComboBoxP.h,v $
 * Revision 1.1.6.1  1994/04/25  17:06:59  drk
 * 	CR 8782: Add real render_table resources.
 * 	[1994/04/25  16:51:48  drk]
 *
 * Revision 1.1.4.10  1993/11/19  16:35:25  drk
 * 	CR 6983: Rework SetValues geometry management.
 * 	[1993/11/19  16:34:59  drk]
 * 
 * Revision 1.1.4.9  1993/11/10  21:10:14  drk
 * 	Minor cleanups.
 * 	[1993/11/10  21:09:44  drk]
 * 
 * Revision 1.1.4.8  1993/11/09  18:10:46  drk
 * 	Added text_changed field to suppress redundant callbacks.
 * 	[1993/11/09  18:09:49  drk]
 * 
 * Revision 1.1.4.7  1993/10/28  23:44:58  drk
 * 	Eliminate static variables.
 * 	[1993/10/28  23:44:16  drk]
 * 
 * Revision 1.1.4.6  1993/10/26  20:42:33  drk
 * 	Let GrabShell handle click-to-post.
 * 	[1993/10/26  20:42:12  drk]
 * 
 * Revision 1.1.4.5  1993/10/21  15:57:57  drk
 * 	Don't redeclare _XmTraverseNextTabGroup or _XmTraversePriorTabGroup.
 * 	[1993/10/21  15:57:44  drk]
 * 
 * Revision 1.1.4.4  1993/10/20  21:36:18  drk
 * 	Distinguish BClick from press & release on the arrow.
 * 	[1993/10/20  21:35:53  drk]
 * 
 * Revision 1.1.4.3  1993/10/14  19:32:47  drk
 * 	CR 6356: Redisplay highlight too.
 * 	[1993/10/14  19:32:26  drk]
 * 
 * Revision 1.1.4.2  1993/09/29  15:57:33  drk
 * 	Cleanup old resources, misc.
 * 	[1993/09/29  15:56:28  drk]
 * 
 * Revision 1.1.2.8  1993/07/23  02:48:44  yak
 * 	Expended copyright marker
 * 	[1993/07/23  01:14:27  yak]
 * 
 * Revision 1.1.2.7  1993/07/08  20:35:33  rgcote
 * 	Latest code drop from Cindy.
 * 	[1993/07/08  20:33:49  rgcote]
 * 
 * Revision 1.1.2.6  1993/06/28  21:55:38  rgcote
 * 	Latest code drop from Cindy.
 * 	[1993/06/28  21:52:31  rgcote]
 * 
 * Revision 1.1.2.5  1993/05/20  18:31:52  rgcote
 * 	Integrated the fourth code drop from Cindy Romeo.
 * 	[1993/05/20  18:27:00  rgcote]
 * 
 * Revision 1.1.2.4  1993/05/10  14:50:53  rgcote
 * 	Integration of third code drop.
 * 	[1993/05/10  14:42:47  rgcote]
 * 
 * Revision 1.1.2.3  1993/04/22  21:47:37  rgcote
 * 	Integration from second code drop.
 * 	[1993/04/20  20:32:10  rgcote]
 * 
 * Revision 1.1.2.2  1993/03/30  22:48:06  motiflotus
 * 	Initial integration.
 * 	[1993/03/30  21:21:49  motiflotus]
 * 
 * $EndLog$
 */
/*	ComboBoxP.h  */
#ifndef _XmComboBoxP_H
#define _XmComboBoxP_H

#include <Xm/ManagerP.h>
#include <Xm/ComboBox.h> 


#ifdef __cplusplus
extern "C" {
#endif

/* New fields for the ComboBox widget class record. */

typedef struct _XmComboBoxClassPart {
  XtPointer extension;		/* Pointer to extension record. */
} XmComboBoxClassPart;


/* Full class record declaration. */	
typedef struct _XmComboBoxClassRec {
  CoreClassPart		core_class;
  CompositeClassPart	composite_class;
  ConstraintClassPart	constraint_class;
  XmManagerClassPart	manager_class;
  XmComboBoxClassPart	combo_box_class;
} XmComboBoxClassRec;

externalref XmComboBoxClassRec xmComboBoxClassRec;

/*
 * New fields for the ComboBox widget record.	
 */

typedef struct _XmComboBoxPart {
  /* Resources */
  unsigned char 	type;
  unsigned char		match_behavior;
  Dimension 		highlight_thickness;
  Dimension 		arrow_size;
  Dimension 		arrow_spacing;
  Dimension 		margin_width;
  Dimension 		margin_height;
  XtCallbackList	selection_callback;
  XmString		selected_item;
  int			selected_position;
  XmFontList		render_table;

  /* Internal data */
  Widget		list_shell; 
  Widget		list; 
  Widget		scrolled_w; 
  Widget		vsb;
  Widget		hsb;
  int 			ideal_ebheight;
  int			ideal_ebwidth;
  GC 			arrow_GC;
  XRectangle 		hit_rect;
  Dimension 		arrow_shadow_width;
  Boolean 		arrow_pressed;
  Boolean		highlighted;
  Boolean		scrolling;
  XtEnum		shell_state;
  Boolean		text_changed;
} XmComboBoxPart;


/* Full instance record declaration. */	

typedef struct _XmComboBoxRec {
  CorePart		core;
  CompositePart		composite;
  ConstraintPart	constraint;
  XmManagerPart		manager;
  XmComboBoxPart 	combo_box;
} XmComboBoxRec;


/********    Private Function Declarations    ********/
#ifdef _NO_PROTO

#else

#endif /* _NO_PROTO */
/********    End Private Function Declarations    ********/


/* Access macros */
#define CB_ArrowPressed(w)   (((XmComboBoxWidget)(w))->combo_box.arrow_pressed)
#define CB_ArrowSize(w)	     (((XmComboBoxWidget)(w))->combo_box.arrow_size)
#define CB_ArrowSpacing(w)   (((XmComboBoxWidget)(w))->combo_box.arrow_spacing)
#define CB_EditBox(w) 	     (((XmComboBoxWidget)(w))->composite.children[0])
#define CB_HighlightThickness(w)	\
	(((XmComboBoxWidget)(w))->combo_box.highlight_thickness)
#define CB_Highlighted(w)    (((XmComboBoxWidget)(w))->combo_box.highlighted)
#define CB_HitRect(w)	     (((XmComboBoxWidget)(w))->combo_box.hit_rect)
#define CB_List(w) 	     (((XmComboBoxWidget)(w))->combo_box.list)
#define CB_ListShell(w)      (((XmComboBoxWidget)(w))->combo_box.list_shell)
#define CB_MarginHeight(w)   (((XmComboBoxWidget)(w))->combo_box.margin_height)
#define CB_MarginWidth(w)    (((XmComboBoxWidget)(w))->combo_box.margin_width)
#define CB_MatchBehavior(w)		\
	(((XmComboBoxWidget)(w))->combo_box.match_behavior)
#define CB_RenderTable(w)    (((XmComboBoxWidget)(w))->combo_box.render_table)
#define CB_ScrolledW(w)      (((XmComboBoxWidget)(w))->combo_box.scrolled_w)
#define CB_SelectedItem(w)   (((XmComboBoxWidget)(w))->combo_box.selected_item)
#define CB_SelectedPosition(w)		\
	(((XmComboBoxWidget)(w))->combo_box.selected_position)
#define CB_SelectionCB(w)		\
	(((XmComboBoxWidget)(w))->combo_box.selection_callback)
#define CB_ShellState(w)     (((XmComboBoxWidget)(w))->combo_box.shell_state)
#define CB_TextChanged(w)    (((XmComboBoxWidget)(w))->combo_box.text_changed)
#define CB_Type(w) 	     (((XmComboBoxWidget)(w))->combo_box.type)


#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif


#endif /* _XmComboBoxP_H */
/* DON'T ADD ANYTHING AFTER THIS #endif */
