/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: NavigatorT.h,v $
 * Revision 1.1.8.1  1994/05/04  19:55:57  drk
 * 	Document trait version numbers.
 * 	[1994/05/04  19:52:56  drk]
 *
 * Revision 1.1.6.2  1994/03/07  15:24:16  drk
 * 	Fix missing includes.
 * 	[1994/03/07  15:23:52  drk]
 * 
 * Revision 1.1.6.1  1994/01/26  15:22:35  daniel
 * 	check in new navigator code
 * 	[1994/01/26  15:19:00  daniel]
 * 
 * Revision 1.1.4.2  1993/09/14  21:16:08  inca
 * 	VMS-ify (externaldef/externalref) traits
 * 	[1993/09/14  21:07:19  inca]
 * 
 * Revision 1.1.2.8  1993/07/23  02:57:20  yak
 * 	Expended copyright marker
 * 	[1993/07/23  01:26:27  yak]
 * 
 * Revision 1.1.2.7  1993/04/26  18:42:32  daniel
 * 	add version field
 * 	[1993/04/26  18:41:33  daniel]
 * 
 * Revision 1.1.2.6  1993/04/07  20:46:43  daniel
 * 	guess what! more work on the trait API
 * 	[1993/04/07  20:44:25  daniel]
 * 
 * Revision 1.1.2.5  1993/04/06  20:57:20  daniel
 * 	more Navigator work
 * 	[1993/04/06  20:56:48  daniel]
 * 
 * Revision 1.1.2.4  1993/04/01  21:52:28  daniel
 * 	more work on Navigator/ScrollFrame
 * 	[1993/04/01  21:51:09  daniel]
 * 
 * Revision 1.1.2.3  1993/04/01  20:58:26  daniel
 * 	add scrollFrame to Text and List
 * 	[1993/04/01  20:53:05  daniel]
 * 
 * Revision 1.1.2.2  1993/03/26  18:11:05  daniel
 * 	implement Navigator/ScrollFrame trait
 * 	[1993/03/26  18:09:11  daniel]
 * 
 * $EndLog$
 */
#ifndef _XmNavigatorT_H
#define _XmNavigatorT_H

#include <Xm/Xm.h>

#ifdef __cplusplus
extern "C" {
#endif

externalref XrmQuark XmQTnavigator;

/* Trait structures and typedefs, place typedefs first */


/* this structure is equivalent to an XPoint but in int,
   not in Position, which are short */
typedef struct _TwoDInt {
    int x;
    int y;
} TwoDIntRec, *TwoDInt;


/* this one can be expanded in the future */
typedef struct _XmNavigatorDataRec {
    Mask valueMask ;
    Mask dimMask ;
    TwoDIntRec value;
    TwoDIntRec minimum;
    TwoDIntRec maximum;
    TwoDIntRec slider_size;
    TwoDIntRec increment;
    TwoDIntRec page_increment;
} XmNavigatorDataRec, *XmNavigatorData;

#define NavAllValid             (OxFFFF)
#define NavDimMask		(1L<<0)
#define NavValue  		(1L<<1)
#define NavMinimum              (1L<<2)
#define NavMaximum		(1L<<3)
#define NavSliderSize		(1L<<4)
#define NavIncrement            (1L<<5)
#define NavPageIncrement	(1L<<6)


#ifdef _NO_PROTO

typedef void (*XmNavigatorMoveCBProc)();
typedef void (*XmNavigatorSetValueProc)();
typedef void (*XmNavigatorGetValueProc)();

#else

typedef void (*XmNavigatorMoveCBProc)(Widget nav, 
				      XtCallbackProc moveCB,
				      XtPointer closure,
				      Boolean setunset);
typedef void (*XmNavigatorSetValueProc)(Widget nav, 
					XmNavigatorData nav_data,
					Boolean notify);
typedef void (*XmNavigatorGetValueProc)(Widget nav, 
					XmNavigatorData nav_data);

#endif /* _NO_PROTO */


/* Version 0: initial release. */

typedef struct _XmNavigatorTraitRec {
  int			  version;		/* 0 */
  XmNavigatorMoveCBProc   changeMoveCB;
  XmNavigatorSetValueProc setValue;
  XmNavigatorGetValueProc getValue;
} XmNavigatorTraitRec, *XmNavigatorTrait;


#define NavigDimensionX			(1L<<0)  
#define NavigDimensionY			(1L<<1)  

/* convenience Macros */
#define ACCESS_DIM(mask,field) ((mask & NavigDimensionX)?(field.x):(field.y))

#define ASSIGN_DIM(mask,field,val)	\
  {					\
    if (mask & NavigDimensionX)		\
      (field.x)=(val);			\
    else				\
      (field.y)=(val);			\
  }


#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /* _XmNavigatorT_H */
