/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: NotebookP.h,v $
 * Revision 1.1.9.3  1994/07/13  22:18:06  knight
 * 	Fix CR 9503: handle geom during ConstraintSetValues
 * 	[1994/07/08  20:47:03  knight]
 *
 * Revision 1.1.9.2  1994/06/17  20:26:15  knight
 * 	Fix 8749: Removed notebook.scroll_frame_inited usage,
 * 	and fix NO_PROTO typo.
 * 	[1994/06/17  20:25:34  knight]
 * 
 * Revision 1.1.9.1  1994/06/15  15:50:17  knight
 * 	Fix CR 8749: Introduction of new spiral binding
 * 	[1994/06/15  15:41:17  knight]
 * 
 * Revision 1.1.7.4  1994/04/19  18:21:16  knight
 * 	Fix for CR5906 and CR6652 - code cleanup.
 * 	[1994/04/19  16:40:18  knight]
 * 
 * 	Fixes for CR6652.
 * 
 * Revision 1.1.7.3  1994/04/08  14:45:21  knight
 * 	Moved GetNotebookConstraint to Notebook.c
 * 	[1994/04/08  14:15:49  knight]
 * 
 * 	Fix for CR8508, CR8512, CR8514, CR8505, CR8518
 * 	Fix for proper child shadow thickness during tab traversal
 * 	[1994/04/07  23:00:49  knight]
 * 
 * 	CR8404: Fixed HOME/END  behavior for tabs
 * 	Added JoinSide behavior fixes
 * 	[1994/04/07  19:32:35  knight]
 * 
 * 	CR7972: Added awareness to JoinSide trait.
 * 	[1994/04/01  19:20:03  knight]
 * 
 * Revision 1.1.7.2  1994/03/18  18:32:35  knight
 * 	Changed notebook component enumeration to be consistent
 * 	[1994/03/18  18:12:10  knight]
 * 
 * 	CR8252 & 8274: Changed names of internal children to match doc
 * 	[1994/03/18  18:05:12  knight]
 * 
 * Revision 1.1.7.1  1994/01/20  21:09:50  inca
 * 	Merge binding GC:s, merge background GC:s, use Manager
 * 	top/bottom_shadow_GC to draw the frame shadow
 * 	[1994/01/20  21:09:24  inca]
 * 
 * Revision 1.1.4.8  1993/11/12  18:55:23  motifibm
 * 	CR's 6923 6916 6861 6471
 * 	[1993/11/12  18:41:38  motifibm]
 * 
 * Revision 1.1.4.7  1993/11/05  23:17:24  rgcote
 * 	Changed the macro constants _HOME, _END, _NEXT and _PREV
 * 	to be numbers rather than strings so they can be used with
 * 	the RepType facility for handling action parameters.
 * 	[1993/11/05  23:14:23  rgcote]
 * 
 * Revision 1.1.4.6  1993/11/02  23:04:32  motifibm
 * 	Fix for CR 6614
 * 	[1993/11/02  23:01:48  motifibm]
 * 
 * Revision 1.1.4.5  1993/11/01  21:48:16  motifibm
 * 	IBM Beta submission
 * 	[1993/11/01  21:47:25  motifibm]
 * 
 * Revision 1.1.4.4  1993/10/15  13:55:48  deblois
 * 	Yet another drop from Anthony.  Includes fixes for CRs:
 * 	5826, 5911, 5916, 5920, 5921, 5922, 5940, 5941, 5960, and 5961.
 * 	[1993/10/15  13:54:45  deblois]
 * 
 * Revision 1.1.4.3  1993/10/07  22:13:52  deblois
 * 	Another Drop From Anthony.
 * 	[1993/10/07  22:13:31  deblois]
 * 
 * Revision 1.1.4.2  1993/09/15  14:53:26  motifibm
 * 	extern to externalref
 * 	[1993/09/15  14:52:42  motifibm]
 * 
 * Revision 1.1.2.7  1993/07/23  02:57:35  yak
 * 	Expended copyright marker
 * 	[1993/07/23  01:26:52  yak]
 * 
 * Revision 1.1.2.6  1993/07/10  02:22:34  meeks
 * 	Fixed includes for Mrminit.c.  Commented out bogus? RepTypeI.h.
 * 	[1993/07/10  02:12:05  meeks]
 * 
 * Revision 1.1.2.5  1993/06/15  15:52:34  shobana
 * 	Fixes from Anthony Moore to conform to policy on #ifdefs
 * 	[1993/06/15  15:52:12  shobana]
 * 
 * Revision 1.1.2.4  1993/05/06  14:21:20  motifibm
 * 	Color enablements.
 * 	[1993/05/06  14:21:03  motifibm]
 * 
 * Revision 1.1.2.3  1993/05/06  14:04:03  motifibm
 * 	     Color enablements.
 * 	     [1993/05/06  14:03:26  motifibm]
 * 
 * Revision 1.1.2.2  1993/04/05  23:55:17  motifibm
 * 	     IBM Drop 3 - Anthony
 * 	     [1993/04/05  23:52:01  motifibm]
 * 
 * 	          IBM Drop 3 - Anthony
 * 	     [1993/04/05  21:16:42  motifibm]
 * 
 * $EndLog$
 */
#ifndef _XmNotebookP_h
#define _XmNotebookP_h

#include <Xm/XmP.h>
#include <Xm/ManagerP.h>
#include <Xm/ScrollFrameT.h>
#include <Xm/Notebook.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Notebook's constraint info. fields */
typedef struct _XmNotebookConstraintPart
{
    /* resources */
    int page_number;			/* page number */
    unsigned char child_type;		/* notebook child type */
    Boolean resizable;			/* is this child resizable? */

    /* private variables */
    Boolean active;			/* True if the child is active */
} XmNotebookConstraintPart, *XmNotebookConstraint;

typedef struct _XmNotebookConstraintRec
{
    XmManagerConstraintPart manager;
    XmNotebookConstraintPart notebook;
} XmNotebookConstraintRec, *XmNotebookConstraintPtr;


/* New fields for the Notebook widget class record */
typedef struct _XmNotebookClassPart
{
    XtPointer extension;
} XmNotebookClassPart;


/* Full class record declaration */
typedef struct _NotebookClassRec
{
    CoreClassPart core_class;
    CompositeClassPart composite_class;
    ConstraintClassPart constraint_class;
    XmManagerClassPart manager_class;
    XmNotebookClassPart	notebook_class;
} XmNotebookClassRec;

externalref XmNotebookClassRec xmNotebookClassRec;


/* New fields for the Notebook widget instance record */
typedef struct _XmNotebookPart
{
    /* resources */
    int current_page_number;		/* the current page number */
    int first_page_number;		/* the first page number */
    int last_page_number;		/* the last page number */
    unsigned char orientation;		/* notebook orientation */
    unsigned char back_page_pos;	/* position of back pages */
    Cardinal back_page_number;		/* the of back page lines */
    Dimension back_page_size;		/* the thickness of back pages */
    Pixel back_page_foreground;         /* foreground color for bk pgs */
    Pixel back_page_background;         /* background color for bk pgs */
    Pixel frame_background;		/* background color for frame */
    unsigned char binding_type;		/* binding type */
    Pixmap binding_pixmap;		/* pixmap for the binding */
    Pixmap spiral_pixmap;		/* pixmap for the spiral binding */
    Dimension binding_width;		/* the width of the binding */
    Dimension margin_width;		/* horizontal margin between widgets */
    Dimension margin_height;		/* vertical margin between widgets */
    Dimension major_spacing;		/* gap between major tabs */
    Dimension minor_spacing;		/* gap between minor tabs */
    Dimension shadow_thickness;		/* notebook frame shadow thickness */
    XtCallbackList page_change_callback;/* the page change callback */

    /* child widgets */
    Widget scroller;			/* the page scroller widget */
    Widget scroller_child;		/* TextF child of def page scroller */
    Widget next_major;			/* next major tab scroll button */
    Widget prev_major;			/* prev major tab scroll button */
    Widget next_minor;			/* next minor tab scroll button */
    Widget prev_minor;			/* prev minor tab scroll button */

    /* preferred children sizes */
    Dimension real_binding_width;	/* real binding width */
    Dimension real_back_page_number;	/* real back page number */
    Dimension page_width;		/* width of page widgets */
    Dimension page_height;		/* height of page widgets */
    Dimension status_width;		/* width of the status areas */
    Dimension status_height;		/* height of the status  areas */
    Dimension major_width;		/* width of major tabs */
    Dimension major_height;		/* height of major tabs */
    Dimension minor_width;		/* width of minor tabs */
    Dimension minor_height;		/* height of minor tabs */
    Dimension scroller_width;		/* width of the page scroller */
    Dimension scroller_height;		/* height of the page scroller */
    Dimension major_scroller_width;	/* width of major scrollers */
    Dimension major_scroller_height;	/* height of major scroller */
    Dimension minor_scroller_width;	/* width of minor scrollers */
    Dimension minor_scroller_height;	/* height of minor scrollers */
    Dimension frame_width;		/* width of the frame */
    Dimension frame_height;		/* height of the frame */

    /* for layouting tabs */
    Widget first_major;			/* the first major tab */
    Widget old_top_major;		/* the old top major tab */
    Widget top_major;			/* the top major tab */
    Widget last_major;			/* the last major tab */
    Widget first_minor;			/* the first minor tab */
    Widget old_top_minor;		/* the old top minor tab */
    Widget top_minor;			/* the top minor tab */
    Widget last_minor;			/* the last minor tab */
    Widget constraint_child;		/* changing geom during ConstraintSV */

    /* shadow thickness state for current page major and minor tab */
    Dimension major_shadow_thickness;	/* joined major tab shadow thickness */
    Dimension minor_shadow_thickness;	/* joined minor tab shadow thickness */
    Widget major_shadow_child;		/* saved shadow thickness tab */
    Widget minor_shadow_child;		/* saved shadow thickness tab */
    Boolean in_setshadow;		/* setting tab shadow thickness */

    /* extra position information */
    unsigned char major_pos;		/* position of major tabs */
    unsigned char minor_pos;		/* position of minor tabs */
    unsigned char binding_pos;		/* binding position */

    /* other misc. variables */
    unsigned char which_tab;		/* currently active tab type */
    int last_alloc_num;			/* lastly allocated page number */
    unsigned char scroller_status;	/* status of the page scroller */
    unsigned short need_scroller;	/* need for tab scrollers */
    Boolean dynamic_last_page_num;	/* True if using dynamic last page# */
    Boolean in_callback;		/* True if processing a callback */
    GC back_page_gc;                    /* GC for drawing backpages */
    GC frame_gc;			/* GC for drawing frame */
    GC binding_gc;                      /* GC for drawing binding */
    GC foreground_gc;                   /* GC for drawing foreground */
    GC background_gc;                   /* GC for drawing background */

    Boolean first_change_managed;	/* flags 1st call to ChangeManaged */
    XmScrollFrameData scroll_frame_data; /* data for ScrollFrame trait */
} XmNotebookPart;


/* Full instance record declaration */
typedef struct _XmNotebookRec
{
    CorePart core;
    CompositePart composite;
    ConstraintPart constraint;
    XmManagerPart manager;
    XmNotebookPart notebook;
} XmNotebookRec;


/******************************************************************************
 *                                                                            *
 *                         constants & useful macros                          *
 *                                                                            *
 ******************************************************************************/

/* internal child types, must not conflict with XmRNBChildType enum */
#define XmMAJOR_TAB_SCROLLER            12
#define XmMINOR_TAB_SCROLLER            13
#define XmTAB_SCROLLER                  14

#ifdef __cplusplus
}
#endif

#endif /* _XmNotebookP_h */

