/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: SpinBP.h,v $
 * Revision 1.1.6.1  1994/03/18  16:56:05  drand
 * 	Fixed CRs 8245 and 7814.  Added resource XmNdetailShadowThickness.
 * 	Changed to allow maximum and minimum to be the same number.
 * 	[1994/03/18  16:55:37  drand]
 *
 * Revision 1.1.4.7  1993/11/08  22:14:14  drand
 * 	Removed several useless instance fields and added ideal_height and
 * 	ideal_width.  Added access macros from SpinB.c,  some have been
 * 	renamed and all have a SB_ prefix.
 * 	[1993/11/08  22:13:30  drand]
 * 
 * Revision 1.1.4.6  1993/11/03  23:27:27  drand
 * 	Removed unused bits.  Changed instance variable names to OSF standards
 * 	and reorganized for better memory usage.
 * 	[1993/11/03  23:25:58  drand]
 * 
 * Revision 1.1.4.5  1993/10/29  21:37:24  drand
 * 	New SpinB Drop,  also moved RepType and Accelerators to proper destinations
 * 	[1993/10/29  21:36:23  drand]
 * 
 * Revision 1.1.4.3  1993/10/07  15:38:16  drand
 * 	New drop from CA,  also changed all the SpinB names to SpinBox.
 * 	[1993/10/07  15:37:38  drand]
 * 
 * Revision 1.1.4.2  1993/09/02  22:02:46  drand
 * 	New CA drop,  changed name from SpinButton to SpinBox,  and other
 * 	fixes and improvements.
 * 	[1993/09/02  21:58:12  drand]
 * 
 * Revision 1.1.2.22  1993/07/23  03:02:14  yak
 * 	Expended copyright marker
 * 	[1993/07/23  01:34:25  yak]
 * 
 * Revision 1.1.2.21  1993/07/13  15:37:36  drand
 * 	Fix CR 5996.  Fix code/doc discrepancies.  Some cleanups as well.
 * 	Also,  added fixes from CA.
 * 	[1993/07/13  15:33:21  drand]
 * 
 * Revision 1.1.2.20  1993/07/09  20:06:58  shobana
 * 	Latest drop from CA
 * 	[1993/07/09  20:06:39  shobana]
 * 
 * Revision 1.1.2.19  1993/06/28  20:58:14  motifca
 * 	  Alpha fixes...
 * 	[1993/06/28  20:57:02  motifca]
 * 
 * Revision 1.1.2.18  1993/06/25  21:28:19  motifca
 * 	  Change resource names...
 * 	[1993/06/25  21:27:20  motifca]
 * 
 * Revision 1.1.2.17  1993/06/25  20:04:35  motifca
 * 	  Misc. alpha changes...
 * 	[1993/06/25  20:01:19  motifca]
 * 
 * Revision 1.1.2.16  1993/06/25  17:02:59  motifca
 * 	More changes for alpha...
 * 	[1993/06/25  16:53:31  motifca]
 * 
 * Revision 1.1.2.15  1993/06/24  21:43:25  motifca
 * 	Changes for SpinButton alpha...
 * 	[1993/06/24  21:42:26  motifca]
 * 
 * Revision 1.1.2.14  1993/06/23  14:10:31  drand
 * 	Latest and greatest version of SpinButton.
 * 	[1993/06/23  14:09:30  drand]
 * 
 * Revision 1.1.2.10  1993/04/02  19:54:03  motifca
 * 	   SpinButton integration
 * 	     -  changed initial/repeat delay 0 to reflect spec
 * 	     -  added SBLeft and SBRight
 * 	     -  added boundary flag (private) returned in crossed_boundary
 * 	     -  value changed callback changes:
 * 	          - call when position changes
 * 	          - set fields in callback structure
 * 	          - use 'new' reasons XmCR_SPIN_UP and XmCR_SPIN_DOWN
 * 	[1993/04/02  19:15:47  motifca]
 * 
 * Revision 1.1.2.9  1993/04/01  19:06:23  drand
 * 	Changed value to XmString.
 * 	[1993/04/01  19:05:49  drand]
 * 
 * Revision 1.1.2.8  1993/04/01  18:48:33  motifca
 * 	Integrate Spin Button - CA...
 * 	[1993/04/01  18:47:09  motifca]
 * 
 * $EndLog$
 */

#ifndef _SpinBP_h
#define _SpinBP_h

#include <Xm/LabelP.h>
#include <Xm/ManagerP.h>
#include <Xm/SpinB.h>

/*  Spin constraints */

typedef struct _XmSpinBoxConstraint
{
      XmStringTable  values;        /* array of XmString */
      int            position;      /* 1-based position of current selection */
      int            num_values;    /* number of XmString in the array */

      int            minimum_value; /* parameters for autonumeric mode */
      int            maximum_value;
      int            increment_value;
      short	     decimal_points;

      unsigned char  sb_child_type;
      unsigned char  arrow_sensitivity;
} XmSpinBoxConstraintPart, *XmSpinBoxConstraint;

typedef struct _XmSpinBoxConstraintRec
{
  XmManagerConstraintPart manager;
  XmSpinBoxConstraintPart spinBox;
} XmSpinBoxConstraintRec, *XmSpinBoxConstraintPtr;

/*  Spin class structure  */
typedef struct _XmSpinBoxClassPart
{
  XtPointer extension;			/* not used */
} XmSpinBoxClassPart;


/*  Full class record declaration for Spin class  */

typedef struct _XmSpinBoxClassRec
{
  CoreClassPart              core_class;
  CompositeClassPart         composite_class;
  ConstraintClassPart        constraint_class;
  XmManagerClassPart         manager_class;
  XmSpinBoxClassPart         spinBox_class;
} XmSpinBoxClassRec;

externalref XmSpinBoxClassRec xmSpinBoxClassRec;

/*  The Spin instance record  */

typedef struct _XmSpinBoxPart
{
      Dimension	     arrow_size;
      Dimension	     margin_width;
      Dimension	     margin_height;
      Dimension	     spacing;
      unsigned int   initial_delay;
      unsigned int   repeat_delay;
      XtCallbackList modify_verify_cb; /* CB list for new selection */
      XtCallbackList value_changed_cb; /* CB list for new selection */
      XRectangle     up_arrow_rect;
      XRectangle     down_arrow_rect;
      Widget         textw;         /* text widget */
      int	     boundary;
      int	     last_hit;
      int	     make_change;
      Dimension	     ideal_height;
      Dimension	     ideal_width;
      GC	     arrow_gc;
      GC	     insensitive_gc;
      Mask	     dim_mask; 	    /* for the navigator trait */
      XtIntervalId   spin_timer;
      Boolean	     up_arrow_pressed;
      Boolean	     down_arrow_pressed;
      unsigned char  arrow_layout;
      unsigned char  default_arrow_sensitivity;
      Dimension	     detail_shadow_thickness;
} XmSpinBoxPart;


/*  Full instance record declaration  */

typedef struct _XmSpinBoxRec
{
  CorePart              core;
  CompositePart         composite;
  ConstraintPart        constraint;
  XmManagerPart         manager;
  XmSpinBoxPart         spinBox;
} XmSpinBoxRec;

/*  Macros */

#define SB_ChildIsNumeric(w) \
  ((w)->sb_child_type == (unsigned char) XmNUMERIC)

#define SB_GetConstraintRec(w) \
  ((XmSpinBoxConstraint)(&((XmSpinBoxConstraintPtr) (w)->core.constraints)->spinBox))

#define SB_MaxPosition(w) \
  (((w)->maximum_value - (w)->minimum_value) / (w)->increment_value)

#define SB_CurrentValue(w) \
  ((w)->minimum_value + ((w)->position * (w)->increment_value))

#define SB_WithChild(w) \
  ((w)->spinBox.textw != NULL)

#define SB_ChildCount(w) \
  ((w)->composite.num_children)


#endif  /* _SpinBP_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */



