/* 
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
*/ 
/* 
 * HISTORY
 * $Log: TextStrSoP.h,v $
 * Revision 1.5.12.1  1994/06/01  21:11:53  drk
 * 	CR 6873: Resolve P.h name conflicts.
 * 	[1994/06/01  16:58:08  drk]
 *
 * Revision 1.5.10.2  1994/03/04  22:27:51  drk
 * 	Hide _Xm routines from public view.
 * 	[1994/03/04  22:01:19  drk]
 * 
 * Revision 1.5.10.1  1994/02/18  22:34:48  inca
 * 	Remodularization of Text & TextField
 * 	[1994/02/18  22:25:38  inca]
 * 
 * Revision 1.5.7.6  1993/09/27  19:56:18  inca
 * 	Fixed the code that moves the primary selection in Text & textField
 * 	 *      CR:s 5637 & 5656. This was facilitated by a change to UTM.
 * 	[1993/09/27  19:54:21  inca]
 * 
 * Revision 1.5.7.5  1993/08/26  20:30:15  inca
 * 	Enabled Text & TextField to retake ownership as follows:
 * 	(per Ellis' specifications)
 * 	Take ownership
 * 	1. If I do not have the selection
 * 	2. If selection is empty
 * 	3. If I lost focus since last time I took ownership
 * 	[1993/08/25  20:00:00  inca]
 * 
 * Revision 1.5.7.4  1993/08/23  22:13:08  yak
 * 	Expanded copyright marker
 * 	[1993/08/23  21:25:12  yak]
 * 
 * Revision 1.5.7.3  1993/07/27  19:29:06  inca
 * 	Keep a couple parameters for BC
 * 	[1993/07/27  19:18:37  inca]
 * 
 * Revision 1.5.7.2  1993/07/21  18:39:02  inca
 * 	Fixes from Motif1.2.3. Rev. 1.5.5.2, ancestor 1.5.2.3, file TextStrSoP.h.
 * 	[1993/07/21  17:43:14  inca]
 * 
 * Revision 1.5.4.2  1993/07/10  02:20:06  meeks
 * 	Moved XmTextStatus to Xm.h.
 * 	[1993/07/10  02:08:30  meeks]
 * 
 * Revision 1.5.2.3  1992/07/13  17:04:00  al
 * 	HP Drop 71
 * 	[1992/07/13  16:59:56  al]
 * 
 * Revision 1.5.2.2  1992/05/13  20:09:03  al
 * 	HP Drop 39
 * 	[1992/05/13  20:06:03  al]
 * 
 * Revision 1.5  1992/03/13  16:51:01  devsrc
 * 	Converted to ODE
 * 
 * $EndLog$
*/ 
/* $RCSfile: TextStrSoP.h,v $ $Revision: 1.5.12.1 $ $Date: 1994/06/01 21:11:53 $ */
/* (c) Copyright 1989, DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS. */
/* (c) Copyright 1987, 1988, 1989, 1990, 1991, 1992 HEWLETT-PACKARD COMPANY */
#ifndef _XmTextStrSoP_h
#define _XmTextStrSoP_h

#include <Xm/XmP.h>
#include <Xm/Text.h>

#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************
 *
 * Definitions for use by sources and source users.
 *
 ****************************************************************/

typedef struct _XmSourceDataRec {
  XmTextSource source;	/* Backpointer to source record. */
  XmTextWidget *widgets;	/* Array of widgets displaying this source. */
  XmTextPosition left, right; /* Left and right extents of selection. */
  char * ptr;			/* Actual string data. */
  char * value;		/* Value of the string data. */
  char * gap_start;		/* Gapped buffer start pointer */
  char * gap_end;		/* Gapped buffer end pointer */
  char * PSWC_NWLN;           /* Holder for char*, short*, int* rep of NWLN */
  int length;			/* Number of chars of data. */
  int maxlength;		/* Space allocated. */
  int old_length;		/* Space allocated for value pointer. */
  int numwidgets;		/* Number of entries in above. */
  int maxallowed;		/* The user is not allowed to grow source */
  /* to a size greater than this. */
  Time prim_time;             /* time of primary selection */
  Boolean hasselection;	/* Whether we own the selection. */
  Boolean editable;		/* Whether we allow any edits. */
  Boolean take_selection;	/* Whether we should take the selection. */
} XmSourceDataRec, *XmSourceData;

#ifdef _NO_PROTO
typedef void (*AddWidgetProc)(); /* source, widget */
#else
typedef void (*AddWidgetProc)(XmTextSource,
			      XmTextWidget);
#endif

#ifdef _NO_PROTO
typedef int (*CountLinesProc)(); /* source, start, length */
#else
typedef int (*CountLinesProc)(XmTextSource,
			      XmTextPosition,
			      unsigned long);
#endif

#ifdef _NO_PROTO
typedef void (*RemoveWidgetProc)(); /* source, widget */
#else
typedef void (*RemoveWidgetProc)(XmTextSource,
				 XmTextWidget);
#endif

#ifdef _NO_PROTO
typedef XmTextPosition (*ReadProc)(); /* source, position,
					 last_position, block */
#else
typedef XmTextPosition (*ReadProc)(XmTextSource,
				   XmTextPosition,	/* starting position */
				   XmTextPosition,	/* The last position 
							   we're interested in.
							   Don't return info 
							   about any later
							   positions. */
				   XmTextBlock);	/* RETURN: text read */
#endif

#ifdef _NO_PROTO
typedef XmTextStatus (*ReplaceProc)(); /* initiator, event, start, end, block,
					  call_callback */
#else
typedef XmTextStatus (*ReplaceProc)(XmTextWidget,
				    XEvent *,
				    XmTextPosition *,
				    XmTextPosition *,
				    XmTextBlock,
#if NeedWidePrototypes
				    int);
#else
                                    Boolean);
#endif /* NeedsWidePrototypes */
#endif

#ifdef _NO_PROTO
typedef XmTextPosition (*ScanProc)(); /* source, pos, sType, dir, 
					 count, include */
#else
typedef XmTextPosition (*ScanProc)(XmTextSource,
				   XmTextPosition,
				   XmTextScanType,
				   XmTextScanDirection,	/*XmsdLeft/XmsdRight*/
				   int,
#if NeedWidePrototypes
				   int);
#else
    		                   Boolean);
#endif /* NeedsWidePrototypes */
#endif

#ifdef _NO_PROTO
typedef Boolean (*GetSelectionProc)(); /* source, left, right */
#else
typedef Boolean (*GetSelectionProc)(XmTextSource,
                                    XmTextPosition *,
                                    XmTextPosition *);
#endif

#ifdef _NO_PROTO
typedef void (*SetSelectionProc)(); /* source, left, right, time */
#else
typedef void (*SetSelectionProc)(XmTextSource,
				 XmTextPosition,
				 XmTextPosition,
				 Time);
#endif


typedef struct _XmTextSourceRec {
  struct _XmSourceDataRec *data;   /* Source-defined data (opaque type). */
  AddWidgetProc	AddWidget;
  CountLinesProc	CountLines;
  RemoveWidgetProc	RemoveWidget;
  ReadProc		ReadSource;
  ReplaceProc		Replace;
  ScanProc		Scan;
  GetSelectionProc	GetSelection;
  SetSelectionProc	SetSelection;
} XmTextSourceRec;


#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /*  _XmTextStrSoP_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */
