/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: Transfer.h,v $
 * Revision 1.1.6.2  1994/04/06  22:28:34  drand
 * 	Fixed SNAPSHOT support and removed XmTransferSetup
 * 	[1994/04/06  22:28:07  drand]
 *
 * Revision 1.1.6.1  1994/01/21  22:06:50  drand
 * 	Remove XmTransferValues,  added XmTransferSetParameters,
 * 	XmTransferStartRequest and XmTransferSendRequest.  Changed
 * 	XmTransferValue to not take parameters and added parm_type
 * 	to XmConvertCallbackStruct.
 * 	[1994/01/21  21:58:11  drand]
 * 
 * Revision 1.1.4.4  1993/10/18  21:51:06  drand
 * 	Removed the flag argument from XmTransferValue and
 * 	XmTransferValues.
 * 	[1993/10/18  19:55:16  drand]
 * 
 * Revision 1.1.4.3  1993/08/30  20:58:37  drand
 * 	Changed convertCallbackStruct -> flag to flags.  Added
 * 	XmCONVERTING_PARTIAL to the flags list for this field.
 * 	[1993/08/30  20:26:01  drand]
 * 
 * Revision 1.1.2.2  1993/02/16  19:26:03  drand
 * 	Initial revision
 * 	[1993/02/16  19:25:47  drand]
 * 
 * $EndLog$
 */

#ifndef _XmTransfer_H
#define _XmTransfer_H

#include <Xm/DragDrop.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Proc typedefs */

#define XmConvertCallbackProc 		XtCallbackProc
#define XmSelectionDoneProc   		XtSelectionDoneProc
#define XmCancelSelectionProc 		XtCancelConvertSelectionProc
#define XmDestinationCallbackProc	XtCallbackProc
#define XmSelectionCallbackProc		XtSelectionCallbackProc

/* Flags */

typedef enum { XmTRANSFER_DONE_SUCCEED = 0, XmTRANSFER_DONE_FAIL, 
	       XmTRANSFER_DONE_CONTINUE, XmTRANSFER_DONE_DEFAULT 
	     } XmTransferStatus;

enum { XmMOVE = XmDROP_MOVE, XmCOPY = XmDROP_COPY, 
       XmLINK = XmDROP_LINK, XmOTHER };

enum { XmSELECTION_DEFAULT = 0, XmSELECTION_INCREMENTAL,
       XmSELECTION_PERSIST, XmSELECTION_SNAPSHOT,
       XmSELECTION_TRANSACT };

enum { XmCONVERTING_NONE = 0, 
       XmCONVERTING_SAME = 1, 
       XmCONVERTING_TRANSACT = 2,
       XmCONVERTING_PARTIAL = 4 };

enum { XmCONVERT_DEFAULT = 0, XmCONVERT_MORE, 
       XmCONVERT_MERGE, XmCONVERT_REFUSE, XmCONVERT_DONE };

/* Callback structures */

typedef struct {
	int		reason;
	XEvent		*event;
	Atom		selection;
	Atom		target;
	XtPointer	source_data;
	XtPointer	location_data;
	int		flags;
	XtPointer	parm;
	int		parm_format;
	unsigned long	parm_length;
	Atom		parm_type;
	int		status;
	XtPointer	value;
	Atom		type;
	int		format;
	unsigned long	length;
} XmConvertCallbackStruct;

typedef struct {
	int		reason;
  	XEvent		*event;
	Atom		selection;
	XtEnum		operation;	
	int		flags;
	XtPointer	transfer_id;
	XtPointer	destination_data;
	XtPointer	location_data;
	Time		time;
} XmDestinationCallbackStruct;

typedef struct {
	int		reason;
  	XEvent		*event;
	Atom		selection;
	Atom		target;
	Atom		type;
	XtPointer	transfer_id;
	int		flags;
	int		remaining;
	XtPointer	value;
	unsigned long	length;
	int		format;
} XmSelectionCallbackStruct;

typedef struct {
	int		reason;
  	XEvent		*event;
	Atom		selection;
	XtPointer	transfer_id;
	XmTransferStatus	status;
	XtPointer	client_data;
} XmTransferDoneCallbackStruct;

#ifdef _NO_PROTO
typedef void (*XmSelectionFinishedProc)();
#else
typedef void (*XmSelectionFinishedProc)(Widget, XtEnum,
					XmTransferDoneCallbackStruct*);
#endif /* _NO_PROTO */

#ifdef _NO_PROTO
void XmTransferDone();
void XmTransferValue();
void XmTransferSetParameters();
void XmTransferStartRequest();
void XmTransferSendRequest();
#else
void XmTransferDone(XtPointer, XmTransferStatus);
void XmTransferValue(XtPointer, Atom, XtCallbackProc,
		     XtPointer,	Time);
void XmTransferSetParameters(XtPointer, XtPointer, int,	unsigned long, Atom);
void XmTransferStartRequest(XtPointer);
void XmTransferSendRequest(XtPointer, Time);
#endif /* _NO_PROTO */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /* _XmTransfer_H */
