/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: TransferI.h,v $
 * Revision 1.1.8.1  1994/06/29  20:41:15  drk
 * 	CR 9539: Fix SGI 64-bit warnings.
 * 	[1994/06/29  20:13:03  drk]
 *
 * Revision 1.1.6.8  1994/04/11  15:25:26  drand
 * 	Cleaned up SNAPSHOT support
 * 	[1994/04/11  15:25:06  drand]
 * 
 * Revision 1.1.6.7  1994/04/06  22:28:31  drand
 * 	Fixed SNAPSHOT support and removed XmTransferSetup
 * 	[1994/04/06  22:28:06  drand]
 * 
 * Revision 1.1.6.6  1994/03/24  20:39:42  drand
 * 	Fix name of XmeTransferGet to _XmTransferGet
 * 	[1994/03/24  20:38:59  drand]
 * 
 * Revision 1.1.6.5  1994/03/04  22:28:13  drk
 * 	Hide _Xm routines from public view.
 * 	[1994/03/04  22:01:37  drk]
 * 
 * Revision 1.1.6.4  1994/01/21  22:06:51  drand
 * 	Added flag for in multiple (TC_IN_MULTIPLE).
 * 	[1994/01/21  21:58:37  drand]
 * 
 * Revision 1.1.6.3  1994/01/11  20:50:31  drand
 * 	Removed Drop Transfer code from UTM
 * 	[1994/01/11  20:49:59  drand]
 * 
 * Revision 1.1.6.2  1994/01/11  14:06:14  drand
 * 	New R6 based Selection code
 * 	[1994/01/11  14:00:11  drand]
 * 
 * Revision 1.1.2.2  1993/02/16  19:16:37  drand
 * 	Initial revision
 * 	[1993/02/16  19:15:35  drand]
 * 
 * $EndLog$
 */
/* $RCSfile: TransferI.h,v $ $Revision: 1.1.8.1 $ $Date: 1994/06/29 20:41:15 $ */
#ifndef _XmTransferI_H
#define _XmTransferI_H

#include "XmSelectI.h"
#include <Xm/TransferT.h>

#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************/
/* Structure which is stored for each transfer request.  This   */
/* is where XmTransferValue will keep the data for the internal */
/* wrapper. 							*/
/****************************************************************/

#define TB_NONE		    0
#define TB_IGNORE	    1
#define TB_INTERNAL   	    2

typedef struct {
  XtPointer		next;
  XtPointer		client_data;
  XtPointer		location_data;
  int			flags;
  Atom			target;
  XtCallbackProc	selection_proc;
} TransferBlockRec, *TransferBlock;

/****************************************************************/
/* This structure forms the context block for each transfer_id. */
/* These structures are chained to allow for multiple           */
/* concurrent outstanding data transfers.                       */
/****************************************************************/

typedef struct {
  XtPointer	next;
  XtPointer	prev;
  Widget	widget;
  Atom		selection;
  Atom		real_selection;
  XtEnum	op;
  int		count;
  int		outstanding;
  int		flags;
  int		status;
  Widget	drag_context;
  Widget	drop_context;
  XmSelectionFinishedProc	*doneProcs;
  int		numDoneProcs;
  XtCallbackProc		auto_proc;
  XtPointer	client_data;
  XmDestinationCallbackStruct	*callback_struct;
  TransferBlock last;
  TransferBlock requests;
} TransferContextRec, *TransferContext;

enum{ TC_NONE = 0, TC_FLUSHED = 1, TC_CALLED_WIDGET = 2,
      TC_CALLED_CALLBACKS = 4, TC_EXITED_DH = 8, 
      TC_DID_DELETE = 16, TC_IN_MULTIPLE = 32 };

/****************************************************************/
/* The convertProc has a small context block which is           */
/* setup by the source calls and deleted in the loseProc        */
/****************************************************************/

typedef struct {
  long		op;	/* Make it big so it can hold a variety of data */
  int		flags;
  long		itemid;
  XtPointer	location_data;
  XtPointer	client_data;
  Widget	drag_context;
} ConvertContextRec, *ConvertContext;

enum{ CC_NONE = 0};

/****************************************************************/
/* Need to keep a count of outstanding SNAPSHOT requests.       */
/****************************************************************/

typedef struct {
  long		outstanding;
  Atom		distinguisher;
} SnapshotRequestRec, *SnapshotRequest;

/* Internal functions */
#ifdef _NO_PROTO

extern char * _XmTextToLocaleText();
extern void _XmConvertHandlerSetLocal();
extern Boolean _XmConvertHandler();
extern Boolean _XmDestinationHandler();
extern void _XmConvertComplete();
extern XmDestinationCallbackStruct *_XmTransferGetDestinationCBStruct();

#else

extern char * _XmTextToLocaleText(Widget, XtPointer, Atom, int,
				  unsigned long, Boolean *);

extern void _XmConvertHandlerSetLocal(void);

extern Boolean _XmConvertHandler(Widget wid, Atom *selection, Atom *target, 
			  Atom *type, XtPointer *value, 
			  unsigned long *size, int *fmt);

extern Boolean _XmDestinationHandler(Widget wid, Atom selection, XtEnum op,
			      XmSelectionFinishedProc done_proc,
			      XtPointer location_data, Time time,
			      XSelectionRequestEvent *event);

extern void _XmConvertComplete(Widget wid, XtPointer value, 
			unsigned long size, int format, Atom type,
			XmConvertCallbackStruct *cs);

extern XmDestinationCallbackStruct
       *_XmTransferGetDestinationCBStruct(XtPointer);
#endif /* _NO_PROTO */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /* _XmTransferI_H */

