/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (MOTIF).  See the file named COPYRIGHT.MOTIF
 * for the full copyright text.
 * 
 */
/*
 * HISTORY
 * $Log: XmRenderTI.h,v $
 * Revision 1.1.4.4  1994/07/18  20:46:10  meeks
 * 	CR 9208: Added declaration for _XmRenderTableRemoveRenditions.
 * 	[1994/07/18  20:42:18  meeks]
 *
 * Revision 1.1.4.3  1994/07/15  13:43:26  meeks
 * 	CR 7628: Forgot _NO_PROTO for _XmRenderTableFindFirstFont.
 * 	[1994/07/15  13:34:18  meeks]
 * 
 * Revision 1.1.4.2  1994/07/14  19:50:15  meeks
 * 	CR 7628: Added _XmRenderTableFindFallback and _XmRenderTableFindFirstFont.
 * 	[1994/07/14  19:47:59  meeks]
 * 
 * Revision 1.1.4.1  1994/06/17  20:59:45  drand
 * 	Fixed ANSI warnings and errors
 * 	[1994/06/17  20:42:42  drand]
 * 
 * Revision 1.1.2.7  1994/04/07  21:05:53  meeks
 * 	CR 8348: Added in_db argument to _XmRenditionCreate.
 * 	[1994/04/06  19:20:05  meeks]
 * 
 * Revision 1.1.2.6  1994/03/29  22:20:22  meeks
 * 	PERF: Added handles to renditions, still leaks.
 * 	[1994/03/28  21:59:38  meeks]
 * 
 * 	PERF: Intermediate check-in of rendition/rendertable sharing.  Leaks and no handles for renditions.
 * 	[1994/03/25  22:03:09  meeks]
 * 
 * Revision 1.1.2.5  1994/03/16  00:10:08  inca
 * 	Fix crash in CSText15 - multiple free:s
 * 	[1994/03/16  00:01:12  inca]
 * 
 * Revision 1.1.2.4  1994/03/07  22:22:34  meeks
 * 	Fixed merge problem?
 * 	[1994/03/07  22:22:20  meeks]
 * 
 * Revision 1.1.2.3  1994/03/07  21:53:30  meeks
 * 	Merged with changes from 1.1.2.2
 * 	[1994/03/07  21:53:18  meeks]
 * 
 * 	Merge.
 * 	[1994/03/07  21:52:09  meeks]
 * 
 * 	Added _XmRenditionCopy for CSText.
 * 	[1994/03/07  21:25:09  meeks]
 * 
 * Revision 1.1.2.2  1994/03/04  22:29:03  drk
 * 	Hide _Xm routines from public view.
 * 	[1994/03/04  22:02:20  drk]
 * 
 * Revision 1.1.2.2  1994/03/04  22:29:03  drk
 * 	Hide _Xm routines from public view.
 * 	[1994/03/04  22:02:20  drk]
 * 
 * Revision 1.1.2.1  1994/03/01  22:01:19  meeks
 * 	Newly created to deal with _Xm functions.
 * 	[1994/03/01  22:00:15  meeks]
 * 
 * $EndLog$
 */
/*   $RCSfile: XmRenderTI.h,v $ $Revision: 1.1.4.4 $ $Date: 1994/07/18 20:46:10 $ */
#ifndef _XmRenderTI_h
#define _XmRenderTI_h

#include <Xm/XmP.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Internal types for XmRenderTable.c */

#define REND_OPTIMIZED_BITS	1
#define REND_MARK_BITS		REND_OPTIMIZED_BITS
#define REND_REFCOUNT_BITS	(16 - REND_OPTIMIZED_BITS)

/*
 * Macros for Rendition data structure access
 */

#define _XmRendRefcount(r)	((_XmRendition)*(r))->refcount
#define _XmRendFontOnly(r)	((_XmRendition)*(r))->fontOnly
#define _XmRendLoadModel(r)	((_XmRendition)*(r))->loadModel
#define _XmRendTag(r)		((_XmRendition)*(r))->tag
#define _XmRendFontName(r)	((_XmRendition)*(r))->fontName
#define _XmRendFontType(r)	((_XmRendition)*(r))->fontType
#define _XmRendFont(r)		((_XmRendition)*(r))->font
#define _XmRendDisplay(r)	((_XmRendition)*(r))->display
#define _XmRendTabs(r)		((_XmRendition)*(r))->tabs
#define _XmRendBG(r)		((_XmRendition)*(r))->background
#define _XmRendFG(r)		((_XmRendition)*(r))->foreground
#define _XmRendBGState(r)	((_XmRendition)*(r))->backgroundState
#define _XmRendFGState(r)	((_XmRendition)*(r))->foregroundState
#define _XmRendUnderlineType(r)	((_XmRendition)*(r))->underlineType
#define _XmRendStrikethruType(r)((_XmRendition)*(r))->strikethruType
#define _XmRendGC(r)		((_XmRendition)*(r))->gc
#define _XmRendTags(r)		((_XmRendition)*(r))->tags
#define _XmRendTagCount(r)	((_XmRendition)*(r))->count
#define _XmRendHadEnds(r)	((_XmRendition)*(r))->hadEnds
#define _XmRendRefcountInc(r)	++(((_XmRendition)*(r))->refcount)
#define _XmRendRefcountDec(r)	--(((_XmRendition)*(r))->refcount)

typedef struct __XmRenditionRec
{
  /* flag indicating _XmFontRenditionRec */
  unsigned int	fontOnly : REND_OPTIMIZED_BITS; 
  unsigned int	refcount : REND_REFCOUNT_BITS;
  
  unsigned char	loadModel;
  XmStringTag	tag;
  String	fontName;
  XmFontType	fontType;
  XtPointer	font;
  Display	*display;
  GC		gc;
  XmStringTag	*tags;
  unsigned int	count;
  Boolean	hadEnds;
  
  XmTabList	tabs;
  Pixel		background;
  Pixel		foreground;
  unsigned char	underlineType;
  unsigned char strikethruType;
  unsigned char backgroundState;
  unsigned char foregroundState;
  
} _XmRenditionRec, *_XmRendition;

typedef struct __XmFontRenditionRec
{
  /* flag indicating _XmFontRenditionRec */
  unsigned int	fontOnly : REND_OPTIMIZED_BITS; 
  unsigned int	refcount : REND_REFCOUNT_BITS;

  unsigned char	loadModel;
  XmStringTag	tag;
  String	fontName;
  XmFontType	fontType;
  XtPointer	font;
  Display	*display;
  GC		*gc;
  XmStringTag	*tags;
  unsigned int	count;
} _XmFontRenditionRec, *_XmFontRendition;

/* Accessor macros. */

#define _XmRTCount(rt)		((_XmRenderTable)*(rt))->count
#define _XmRTRenditions(rt)	((_XmRenderTable)*(rt))->renditions
#define _XmRTDisplay(rt)	((_XmRenderTable)*(rt))->display
#define _XmRTMark(rt)		((_XmRenderTable)*(rt))->mark
#define _XmRTRefcount(rt)	((_XmRenderTable)*(rt))->refcount
#define _XmRTRefcountInc(rt)	++(((_XmRenderTable)*(rt))->refcount)
#define _XmRTRefcountDec(rt)	--(((_XmRenderTable)*(rt))->refcount)

#define RENDITIONS_IN_STRUCT	1

typedef struct __XmRenderTableRec
{
  unsigned int			mark : REND_MARK_BITS;
  unsigned int			refcount : REND_REFCOUNT_BITS;
  unsigned short		count;
  Display			*display;
  XmRendition			renditions[RENDITIONS_IN_STRUCT];
} _XmRenderTableRec, 		*_XmRenderTable;


/********    Private Function Declarations for XmRenderTable.c    ********/
#ifdef _NO_PROTO

extern XmRendition _XmRenderTableFindRendition(); 
extern XmRendition _XmRenditionCreate();
extern XmRendition _XmRenderTableGetMerged();
extern XmRendition _XmRenditionMerge(); 
extern Widget _XmCreateRenderTable(); 
extern Widget _XmCreateRendition();
extern Display *_XmRenderTableDisplay();
extern XmRendition _XmRenditionCopy();
extern Boolean _XmRenderTableFindFallback();
extern Boolean _XmRenderTableFindFirstFont();
extern XmRenderTable _XmRenderTableRemoveRenditions();
 
#else

extern XmRendition _XmRenderTableFindRendition(XmRenderTable table, 
					       XmStringTag tag,
#if NeedWidePrototypes
					       int cached_tag,
					       int need_font,
					       int call,
#else
					       Boolean cached_tag,
					       Boolean need_font,
					       Boolean call,
#endif /* NeedWidePrototypes */
					       short *index); 
extern XmRendition _XmRenditionCreate(Display *display,
				      Widget widget,
				      String resname,
				      String resclass,
				      XmStringTag tag,
				      ArgList arglist,
				      Cardinal argcount,
				      Boolean *in_db); 
extern XmRendition _XmRenderTableGetMerged(XmRenderTable rt,
					   XmStringTag base,
					   XmStringTag *tags,
#if NeedWidePrototypes
					   unsigned int tag_count 
#else			
                                           unsigned short tag_count 
#endif /* NeedWidePrototypes */
					   );
extern XmRendition _XmRenditionMerge(Display *d,
				     XmRendition *scr,
				     XmRendition base_rend,
				     XmRenderTable rt,
				     XmStringTag base_tag,
				     XmStringTag *tags,
#if NeedWidePrototypes
				     unsigned int tag_count,
                                     unsigned int copy
#else			
				     unsigned short tag_count,
                                     Boolean copy
#endif /* NeedWidePrototypes */
				     ); 
extern Widget _XmCreateRenderTable(Widget parent,
				   String name,
				   ArgList arglist,
				   Cardinal argcount); 
extern Widget _XmCreateRendition(Widget parent,
				 String name,
				 ArgList arglist,
				 Cardinal argcount); 
extern Display *_XmRenderTableDisplay(XmRenderTable table);
extern XmRendition _XmRenditionCopy(XmRendition rend,
				    Boolean shared);
extern Boolean _XmRenderTableFindFallback(XmRenderTable ,
					  XmStringTag tag,
#if NeedWidePrototypes
					  int cached_tag,
#else
					  Boolean cached_tag,
#endif /* NeedWidePrototypes */
					  short *indx,
					  XmRendition *rend_ptr) ;
extern Boolean _XmRenderTableFindFirstFont(XmRenderTable rendertable,
					   short *indx,
					   XmRendition *rend_ptr);
extern XmRenderTable _XmRenderTableRemoveRenditions(XmRenderTable oldtable,
						    XmStringTag *tags,
						    int tag_count,
#if NeedWidePrototypes
						    int chk_font,
#else
						    Boolean chk_font,
#endif /* NeedWidePrototypes */
						    XmFontType type,
						    XtPointer font);

#endif /* _NO_PROTO */
/********    End Private Function Declarations    ********/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

#endif /* _XmRenderTI_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */
