/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (MOTIF).  See the file named COPYRIGHT.MOTIF
 * for the full copyright text.
 * 
 */
/*
 * HISTORY
 * $Log: XmSelectI.h,v $
 * Revision 1.1.2.1  1994/01/11  14:06:17  drand
 * 	New R6 based Selection code
 * 	[1994/01/11  14:00:14  drand]
 *
 * $EndLog$
 */

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#ifndef _XMselectionI_h
#define _XMselectionI_h

#include <X11/Intrinsic.h>

typedef struct _RequestRec *Request;
typedef struct _SelectRec *Select;

typedef struct _RequestRec {
   Select ctx;		      /* logical owner */
   Widget widget;	      /* widget actually receiving Selection events */
   Window requestor;
   Atom property;
   Atom target;
   Atom type;
   int format;
   XtPointer value;
   unsigned long bytelength;
   int offset;
   XtIntervalId timeout;
   XSelectionRequestEvent event; /* for XtGetSelectionRequest */
   Boolean allSent;
} RequestRec;

typedef struct {
  Atom prop;
  Boolean avail;
} SelectionPropRec, *SelectionProp;

typedef struct {
  Display *dpy;
  Atom incr_atom, indirect_atom, timestamp_atom;
  int propCount;
  SelectionProp list;
} PropListRec, *PropList;

typedef struct _SelectRec {
    Atom selection; 			/* constant */
    Display *dpy; 			/* constant */
    Widget widget;
    Time time;
    unsigned long serial;
    XtConvertSelectionProc convert;
    XtLoseSelectionProc loses;
    XtSelectionDoneProc notify;
    XtCancelConvertSelectionProc owner_cancel;
    XtPointer owner_closure;
    PropList prop_list;
    Request req;			/* state for local non-incr xfer */
    int ref_count;			/* of active transfers */
    unsigned int incremental:1;
    unsigned int free_when_done:1;
    unsigned int was_disowned:1;
} SelectRec;

typedef struct _ParamRec {
    Atom selection;
    Atom param;
} ParamRec, *Param;

typedef struct _ParamInfoRec {
    unsigned int count;
    Param paramlist;
} ParamInfoRec, *ParamInfo;	

typedef struct _QueuedRequestRec {
    Atom selection;
    Atom target;
    Atom param;
    XtSelectionCallbackProc callback;
    XtPointer closure;
    Time time;
    Boolean incremental;
} QueuedRequestRec, *QueuedRequest;

typedef struct _QueuedRequestInfoRec {
    int count;
    Atom *selections;
    QueuedRequest *requests;
} QueuedRequestInfoRec, *QueuedRequestInfo;

typedef struct {
    XtSelectionCallbackProc *callbacks;
    XtPointer *req_closure;
    Atom property;
    Atom *target;
    Atom type;
    int format;
    char *value;
    int bytelength;
    int offset;
    XtIntervalId timeout;
    XtEventHandler proc;
    Widget widget;
    Time time;
    Select ctx;
    Boolean *incremental;
    int current;
} CallBackInfoRec, *CallBackInfo;

typedef struct {
  Atom target;
  Atom property;
} IndirectPair;

#define IndirectPairWordSize 2

typedef struct {
  int active_transfer_count;
} RequestWindowRec;

#define MAX_SELECTION_INCR(dpy) (((65536 < XMaxRequestSize(dpy)) ? \
	(65536 << 2)  : (XMaxRequestSize(dpy) << 2))-100)

#define MATCH_SELECT(event, info) ((event->time == info->time) && \
	    (event->requestor == XtWindow(info->widget)) && \
	    (event->selection == info->ctx->selection) && \
	    (event->target == *info->target))

/* Internal Functions */
#if (XtSpecificationRelease > 5)
#define _XmGetSelectionValue XtGetSelectionValue
#define _XmGetSelectionValueIncremental XtGetSelectionValueIncremental
#define _XmGetSelectionValues XtGetSelectionValues
#define _XmGetSelectionValuesIncremental XtGetSelectionValuesIncremental
#define _XmReservePropertyAtom XtReservePropertyAtom
#define _XmReleasePropertyAtom XtReleasePropertyAtom
#define _XmCreateSelectionRequest XtCreateSelectionRequest
#define _XmSendSelectionRequest XtSendSelectionRequest
#define _XmCancelSelectionRequest XtCancelSelectionRequest
#define _XmSetSelectionParameters XtSetSelectionParameters
#define _XmGetSelectionParameters XtGetSelectionParameters
#endif /* R6 */

#ifdef _NO_PROTO 
extern void _XmGetSelectionValue();
extern void _XmGetSelectionValueIncremental();
extern void _XmGetSelectionValues();
extern void _XmGetSelectionValuesIncremental();
extern Atom _XmReservePropertyAtom();
extern void _XmReleasePropertyAtom();
extern void _XmCreateSelectionRequest();
extern void _XmSendSelectionRequest();
extern void _XmCancelSelectionRequest();
extern void _XmSetSelectionParameters();
extern void _XmGetSelectionParameters();
#else
extern void _XmGetSelectionValue(Widget, Atom, Atom, 
				 XtSelectionCallbackProc, XtPointer, Time);
extern void _XmGetSelectionValueIncremental(Widget, Atom, Atom, 
					    XtSelectionCallbackProc, 
					    XtPointer, Time);
extern void _XmGetSelectionValues(Widget, Atom, Atom*, int,
				  XtSelectionCallbackProc, XtPointer*, Time);
extern void _XmGetSelectionValuesIncremental(Widget, Atom, Atom*, int,
					     XtSelectionCallbackProc, 
					     XtPointer*, Time);
extern Atom _XmReservePropertyAtom(Widget);
extern void _XmReleasePropertyAtom(Widget, Atom);
extern void _XmCreateSelectionRequest(Widget, Atom);
extern void _XmSendSelectionRequest(Widget, Atom, Time);
extern void _XmCancelSelectionRequest(Widget, Atom);
extern void _XmSetSelectionParameters(Widget, Atom, Atom type,
				      XtPointer, unsigned long,
				      int);
extern void _XmGetSelectionParameters(Widget, Atom, XtRequestId,
				      Atom*, XtPointer*, unsigned long*,
				      int *);
#endif /* _NO_PROTO */

#ifndef LOCK_PROCESS
#define LOCK_PROCESS
#define UNLOCK_PROCESS
#define LOCK_APP(x)
#define UNLOCK_APP(x)
#endif /* LOCK_PROCESS */

#endif /* _XMselectionI_h */
/* DON'T ADD STUFF AFTER THIS #endif */
